# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'easy_type'
require 'puppet/property/boolean'
require 'puppet_x/enterprisemodules/github/access'

Puppet::Type.newtype(:github_collaborator) do
  include EasyType
  include Puppet_X::EnterpriseModules::GitHub

  desc <<-DESCR
  Allows you to manage github collaborators on your repository. For
  organization-owned repositories, the list of collaborators includes outside
  collaborators, organization members with access through team memberships,
  organization members with access through default organization permissions,
  and organization owners.

  Here is an example o how to do this:

      github_collaborator { 'authentication/owner/repo:collaborator':
        ensure      => 'present',
        permissions => 'pull',
      }

  DESCR

  ensurable

  on_prefetch do |resources, provider|
    resources_in(resources, provider) do |repo, credentials|
      collaborators(repo).collect { |e| e.to_hash.merge(:name => "#{credentials}/#{repo}:#{e.login}") }
    end
  end

  to_get_raw_resources do
    values = []
    with_all_credentials do |credentials|
      values << repositories.collect do |repo|
        Puppet.debug "Fetching collaborators for #{repo.full_name}"
        collaborators(repo.full_name).collect { |e| e.to_hash.merge(:name => "#{credentials}/#{repo.full_name}:#{e.login}") }
      end.flatten
    end
    values.flatten
  end

  on_create do
    authenticated_with(credentials) do
      add_collaborator(full_name, collaborator, rights)
    end
  end

  on_modify do
    authenticated_with(credentials) do
      add_collaborator(full_name, collaborator, rights)
    end
  end

  on_destroy do
    authenticated_with(credentials) do
      remove_collaborator(full_name, collaborator)
    end
  end

  map_titles_to_attributes([%r{((.*)/(.*)/(.*):(.*))}, [:name, :credentials, :owner, :repository, :collaborator]])

  parameter :name
  parameter :credentials
  parameter :repository
  parameter :owner
  parameter :collaborator
  property  :permissions
end
