# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'easy_type'
require 'puppet/property/boolean'
require 'puppet/parameter/boolean'
require 'puppet_x/enterprisemodules/github/access'
require 'easy_type/array_property'

Puppet::Type.newtype(:github_hook) do
  include EasyType
  extend EasyType::Encryption
  include Puppet_X::EnterpriseModules::GitHub
  desc <<-DESCR
  Allows you to manage repository hooks. The Repository Webhooks API allows
  repository admins to manage the post-receive hooks for a repository. Webhooks
  can be managed using the JSON HTTP API, or the [PubSubHubbub API](https://developer.github.com/v3/repos/hooks/#pubsubhubbub).

  If you would like to set up a single webhook to receive events from all of
  your organization's repositories, check out the github [API documentation](https://developer.github.com/v3/orgs/hooks/)
  for Organization Webhooks.

  Here is an example:

      github_hook { 'authenticator/owner/repo:web':
        ensure => 'present',
        active => 'true',
        config => {'content_type' => 'form', 'insecure_ssl' => '0', 'url' => 'https://myurl.com'},
        events => ['push'],
      }


  DESCR

  ensurable

  on_prefetch do |resources, provider|
    resources_in(resources, provider) do |repo, credentials|
      hooks(repo).collect { |e| e.to_hash.merge(:name => "#{credentials}/#{repo}:#{e.name}") }
    end
  end

  to_get_raw_resources do
    values = []
    with_all_credentials do |credentials|
      values << repositories.collect do |repo|
        name      = repo[:full_name]
        full_name = "#{credentials}/#{repo[:full_name]}"
        Puppet.debug "Fetching hooks for #{full_name}"
        hooks(name).collect { |e| e.to_hash.merge(:repo => full_name) }
      end.flatten
    end
    values.flatten
  end

  on_create do
    authenticated_with(credentials) do
      create_hook(full_name, hook, config, options)
    end
  end

  on_modify do
    authenticated_with(credentials) do
      modify_hook(full_name, provider.id, hook, config, options)
    end
  end

  on_destroy do
    authenticated_with(credentials) do
      remove_hook(full_name, provider.id)
    end
  end

  map_titles_to_attributes([%r{((.*)/(.*)/(.*):(.*))}, [:name, :credentials, :owner, :repository, :hook]])

  parameter :id
  parameter :name
  parameter :credentials
  parameter :owner
  parameter :repository
  parameter :hook

  property :active
  property :events
  property :config

  private

  def options
    value = {}
    value[:events] = events if events
    value[:active] = active == true
    value
  end
end
