# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'easy_type'
require 'puppet/property/boolean'
require 'puppet/parameter/boolean'
require 'puppet_x/enterprisemodules/github/access'

Puppet::Type.newtype(:github_repo) do
  include EasyType
  include Puppet_X::EnterpriseModules::GitHub
  @module_name = 'github_config'

  desc <<-DESCR
  Allows you to create and manage github repos. This includes repositories owned
  by the authenticated user, repositories where the authenticated user is a
  collaborator, and repositories that the authenticated user has access to
  through an organization membership.

  Here is an example on how to do this:

      github_repo { 'account/owner/repo':
        ensure        => 'present',
        description   => 'Just some text.',
        has_issues    => 'true',
        has_wiki      => 'true',
        homepage      => 'http://www.enterprisemodules.com/',
      }


  DESCR

  ensurable

  to_get_raw_resources do
    values = []
    with_all_credentials do |credentials|
      Puppet.debug 'Fetching repositories...'
      values << repositories.collect { |r| r.to_h.merge!(:name => "#{credentials}/#{r[:full_name]}") }
    end
    values.flatten
  end

  on_create do
    authenticated_with(credentials) do
      create_repository(repository, options)
    end
  end

  on_modify do
    authenticated_with(credentials) do
      modify_repository(full_name, options)
    end
  end

  on_destroy do
    if really_delete
      authenticated_with(credentials) do
        delete_repository(full_name)
      end
    else
      Puppet.warning "Repository #{name} set to absent, but really_delete not true, so not removing repo."
    end
  end

  map_titles_to_attributes([%r{((.*)/(.*)/(.*))}, [:name, :credentials, :owner, :repository]])

  parameter :name
  parameter :credentials
  parameter :owner
  parameter :repository
  parameter :really_delete

  property :description
  property :homepage
  property :private
  property :has_issues
  property :has_wiki

  private

  def options
    value = {}
    value[:organization] = owner if repo_for_organization?
    value[:description] = description if description
    value[:homepage] = homepage if homepage
    value[:private] = private if private
    value[:has_issues] = has_issues if has_issues
    value[:has_wiki] = has_wiki if has_wiki
    value[:default_branch] = default_branch if default_branch
    value
  end
end
