# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'easy_type'
require 'puppet/property/boolean'
require 'puppet/parameter/boolean'
require 'easy_type/array_property'
require 'puppet_x/enterprisemodules/github/access'

Puppet::Type.newtype(:github_team) do
  include EasyType
  include Puppet_X::EnterpriseModules::GitHub

  desc <<-DESCR

  Allows you to manage a github team. Here is an example on how to do this:

      github_team { 'authentication/organization/team_name':
        ensure      => 'present',
        description => 'My first team',
        members     => ['jim', 'spock', 'checkov']
        privacy     => 'secret',
      }

  DESCR

  ensurable

  to_get_raw_resources do
    values = []
    with_all_credentials do |credentials|
      Puppet.debug 'Fetching organizations...'
      values << organizations.collect do |o|
        org_name = o.login
        Puppet.debug "Fetching teams for #{org_name}"
        organization_teams(org_name).collect { |e| e.to_hash.merge(:name => "#{credentials}/#{org_name}/#{e.name}") }
      end
    end
    values.flatten
  end

  on_create do
    authenticated_with(credentials) do
      create_team(organization, team, options)
    end
  end

  on_modify do
    authenticated_with(credentials) do
      update_team(provider.id, options)
    end
  end

  on_destroy do
    authenticated_with(credentials) do
      delete_team(provider.id)
    end
  end

  map_titles_to_attributes([%r{((.*)/(.*)/(.*))}, [:name, :credentials, :organization, :team]])

  parameter :id
  parameter :name
  parameter :credentials
  parameter :organization
  parameter :team

  property :description
  property :privacy
  property :members

  private

  def options
    value = {}
    value[:description] = description if description
    value[:privacy] = privacy if privacy
    value
  end
end
