# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'puppet/type/github_credentials'

module Puppet_X
  module EnterpriseModules
    # All github specific code
    module GitHub
      require 'octokit'

      def self.included(parent)
        parent.extend(GitHub)
      end

      def configuration
        if @configuration.nil? || @configuration == {}
          @configuration = read_from_yaml
        else
          @configuration
        end
      end

      def read_from_yaml
        config_file = Puppet::Type::Github_credentials.config_file
        if File.exist?(config_file)
          File.open(config_file, 'r') { |f| YAML.safe_load(f) }
        else
          {}
        end
      end

      def configuration_for(key)
        fail "No configuration found for #{key}." unless configuration.keys.include?(key)

        configuration[key]
      end

      def configuration_value_for(credentials, value, default_value = '')
        configuration_for(credentials).fetch(value) { default_value }
      end

      def with_all_credentials
        configuration.each_key do |credentials|
          authenticated_with(credentials) do
            yield credentials
          end
        end
      end

      def authenticated_with(credentials)
        properties = configuration[credentials]
        raise 'Invalid credentials identifier specified' if properties.nil?

        extend(EasyType::Encryption)
        Octokit.configure do |config|
          config.password = decrypted_value(properties['password'])
          config.login = properties['username']
        end
        yield
      end

      def resources_in(resources, provider)
        subresources = []
        credentials = resources.keys.collect { |r| r.split('/').first }
        credentials.each do |credential|
          authenticated_with(credential) do
            repo_names = resources_for(resources, credential).collect { |r| r.split(':').first.gsub(%r{^.*?/}, '') }
            subresources << repo_names.collect do |repo|
              yield(repo, credential)
            end
          end
        end
        subresources.flatten!
        resources.each do |name, _value|
          entry = subresources.select { |e| e[:name] == name }.first
          if entry
            value = provider.map_raw_to_resource(entry)
            resources[name].provider = value
          end
        end
      end

      def resources_for(resources, credentials)
        resources.keys.select { |r| r.split('/').first == credentials }
      end

      def organization_teams(organization)
        Octokit.organization_teams(organization)
      end

      def create_team(organization, name, options)
        retval = Octokit.create_team(organization, options.merge(:name => name))
        fail "Error cresating team with #{name}." unless retval
      end

      def update_team(team_id, options)
        retval = Octokit.update_team(team_id, options)
        fail "Error updating team with id #{id}." unless retval
      end

      def delete_team(team_id)
        retval = Octokit.delete_team(team_id)
        fail "Error deleting team with id #{id}." unless retval
      end

      def team_repositories(id)
        Octokit.team_repositories(id)
      end

      def add_team_repository(id, repo, permission)
        retval = Octokit.add_team_repository(id, repo, :permission => permission)
        fail "Error adding repository #{repo} to team." unless retval
      end

      def remove_team_repository(id, repo)
        retval = Octokit.remove_team_repository(id, repo)
        fail "Error removing repository #{repo} from team." unless retval
      end

      def team_members(id)
        Octokit.team_members(id)
      end

      def add_team_membership(id, user)
        retval = Octokit.add_team_membership(id, user)
        fail "Error add team membership for user #{user}." unless retval
      end

      def remove_team_membership(id, user)
        retval = Octokit.remove_team_membership(id, user)
        fail "Error removing team membership for user #{user}." unless retval
      end

      def organizations
        Octokit.organizations
      end

      def get_organization(organization)
        Octokit.organization(organization)
      end

      def modify_organization(organization, options)
        retval = Octokit.update_organization(organization, options)
        fail "Error modifying organization #{organization}." unless retval
      end

      def org_members(organization)
        Octokit.organization_members(organization)
      end

      def deploy_keys(repository)
        Octokit.deploy_keys(repository, {})
      end

      def add_deploy_key(repository, name, key, options)
        retval = Octokit.add_deploy_key(repository, name, key, options)
        fail "Error add deploy key #{name} to #{repository}." unless retval
      end

      def remove_deploy_key(repository, id)
        retval = Octokit.remove_deploy_key(repository, id, {})
        fail "Error removing deploy key #{id} from #{repository}." unless retval
      end

      def repositories
        Octokit.repositories
      end

      def create_repository(repository, options)
        retval = Octokit.create_repository(repository, options)
        fail "Error creating #{repository}." unless retval
      end

      def modify_repository(repository, options)
        retval = Octokit.update_repository(repository, options)
        fail "Error modifying #{repository}." unless retval
      end

      def delete_repository(repository)
        retval = Octokit.delete_repository(repository, {})
        fail "Error deleting #{repository}." unless retval
      end

      def hooks(repository)
        Octokit.hooks(repository)
      end

      def create_hook(repository, hook, config, options)
        retval = Octokit.create_hook(repository, hook, config, options)
        fail "Error creating hook #{hook} on #{repository}." unless retval
      end

      def modify_hook(repository, id, hook, config, options)
        retval = Octokit.edit_hook(repository, id, hook, config, options)
        fail "Error modifying hook #{hook} on #{repository}." unless retval
      end

      def remove_hook(repository, id)
        retval = Octokit.remove_hook(repository, id)
        fail "Error removing #{hook} from #{repository}." unless retval
      end

      def collaborators(repository)
        Octokit.collaborators(repository)
      end

      def add_collaborator(repository, collaborator, rights)
        retval = Octokit.add_collaborator(repository, collaborator, :permission => rights)
        fail "Error adding #{collaborator} to #{repository}." unless retval
      end

      def remove_collaborator(repository, collaborator)
        retval = Octokit.remove_collaborator(repository, collaborator)
        fail "Error removing #{collaborator} to #{repository}." unless retval
      end

      def module_name
        'github_config'
      end

      def repo_for_organisation?
        configuration[credentials]['username'] != owner
      end

      def full_name
        "#{owner}/#{repository}"
      end
    end
  end
end
